package com.example.civilizedcity.controller;


import cn.hutool.core.codec.Base64;
import cn.hutool.core.io.FileUtil;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.OSSUtils;
import com.example.civilizedcity.common.ResponseBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.Map;

@Api(tags ="通用")
@RestController
@RequestMapping("/")
public class CommController extends BaseController {

    @Autowired
    OSSUtils ossUtils;

    @Value("${yz.upload.path}")
    String uploadPath;
    @Value("${yz.upload.prefix}")
    String uploadPrefix;

    @GetMapping("/ma/captcha")
    @ApiOperation(value="获取验证码", notes = "获取验证码", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getCaptcha(@ApiParam("手机号码") @RequestParam("phone") String phone) throws Exception {

        return ResponseBean.success("success");
    }

    /**
     * 文件
     * @param multipartFile
     * @return
     */
    @ApiOperation(value = "文件上传", notes = "文件上传")
    @PostMapping("/{client}/file")
    public ResponseBean uploadImage(@ApiParam("客户端") @PathVariable String client,
                                    @ApiParam("文件") @RequestParam("file") MultipartFile multipartFile,
                                    @ApiParam("文件类型") @RequestParam(value = "fileType", defaultValue = "image") String fileType) throws Exception {

        //获取文件名
        String fileName = multipartFile.getOriginalFilename();
        String newFileName = String.format("%s/%d-%s", getMonth(), System.currentTimeMillis(), fileName);

        // 上传目录以日期分类
        File tmp = new File(String.format("%s/%s", getUploadPath(), getMonth()));
        if (!tmp.exists()) {
            tmp.mkdir();
        }

        //
            File upFile = new File(String.format("%s/%s", getUploadPath(), newFileName));
        multipartFile.transferTo(upFile);

        String url = String.format("%s/%s", uploadPrefix, newFileName);
        return ResponseBean.success(new HashMap<String, Object>(){{
            put("url", url);
            put("fileType", fileType);
        }});
    }


    /**
     * 文件
     * @param base64
     * @return
     */
    @ApiOperation(value = "文件上传", notes = "文件上传")
    @PostMapping("/{client}/file/base64")
    public ResponseBean uploadImageBase64(@ApiParam("客户端") @PathVariable String client,
                                    @ApiParam("文件") @RequestParam("base64") String base64,
                                          @ApiParam("文件名称") @RequestParam("fileName") String fileName,
                                    @ApiParam("文件类型") @RequestParam(value = "fileType", defaultValue = "image") String fileType) throws Exception {

        String base64Str = base64;
        // 去除Base64字符串的前缀（如果有）
        if (base64Str.startsWith("data:")) {
            base64Str = base64Str.substring(base64Str.indexOf("base64,") + 7);
        }

        // 解码Base64字符串并写入文件
        byte[] decodedBytes = Base64.decode(base64Str);
        //获取文件名
        String fName = String.format("%s/%d-%s", getMonth(), System.currentTimeMillis(), fileName);

        // 上传目录以日期分类
        String upPath = getUploadPath();
        File tmp = new File(String.format("%s/%s", upPath, getMonth()));
        if (!tmp.exists()) {
            tmp.mkdir();
        }

        FileUtil.writeBytes(decodedBytes, FileUtil.file(upPath + "/" + fName));
        String url = String.format("%s/%s", uploadPrefix, fName);
        return ResponseBean.success(new HashMap<String, Object>(){{
            put("url", url);
            put("fileType", fileType);
        }});
    }

    private String getUploadPath() {
        File tmp = new File(uploadPath);
        if (!tmp.exists()) {
            tmp.mkdir();
        }

        return uploadPath;
    }

    private String getMonth() {
        LocalDateTime now = LocalDateTime.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM");
        return now.format(formatter);
    }
}
