package com.example.civilizedcity.controller;

import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import cn.binarywang.wx.miniapp.bean.WxMaPhoneNumberInfo;
import cn.binarywang.wx.miniapp.bean.WxMaUserInfo;
import cn.dev33.satoken.stp.StpUtil;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.ResponseBean;
import com.example.civilizedcity.common.StringUtils;
import com.example.civilizedcity.config.WxMaUitl;
import com.example.civilizedcity.entity.SysUser;
import com.example.civilizedcity.entity.TaPerson;
import com.example.civilizedcity.service.SysLoginService;
import com.example.civilizedcity.service.SysUserDutyService;
import com.example.civilizedcity.service.SysUserService;
import com.example.civilizedcity.service.TaPersonReflectService;
import com.example.civilizedcity.vo.LoginParam;
import com.example.civilizedcity.vo.WxChangePwd;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Api(tags = "小程序登入/登出")
@RestController
@RequestMapping("/ma")
public class WxMaController extends BaseController {

    @Autowired
    WxMaUitl wxMaUitl;

    @Autowired
    TaPersonReflectService taPersonReflectService;

    @Autowired
    SysUserDutyService sysUserDutyService;

    @Autowired
    SysLoginService sysLoginService;


//    private boolean checkLoginParam(LoginParam loginParam) {
//        return !StringUtils.isEmpty(loginParam.getCode())
//                && !StringUtils.isEmpty(loginParam.getUserName())
//                && !StringUtils.isEmpty(loginParam.getPassword());
//    }

//    @GetMapping("/preload")
//    @ApiOperation(value="小程序登录", notes = "小程序登录", httpMethod = "POST", response = ResponseBean.class)
//    public ResponseBean preload(@ApiParam("客户端ID") @PathVariable String clientId,
//                                @ApiParam("小程序预加载参数") @RequestParam(required = false) WxMaPreload wxMaPreload) throws Exception {
//
//        if (Constants.CLIENT_WORKER.equals(clientId)) {
//            List<TaMessageTemplate> list = iTaMessageTemplateService.getByTplType(Constants.MESSAGE_MINIAPP);
//            return ResponseBean.success(new HashMap<String, Object>(){{
//                put("tpls", list);
//            }});
//        }
//
//        return ResponseBean.success(null);
//    }

    /**
     * 小程序
     *
     * @param code
     * @return
     * @throws Exception
     */
    @PostMapping("/login")
    @ApiOperation(value = "小程序登录", notes = "小程序登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean login(@ApiParam("登录参数") @RequestParam String code) throws Exception {

        if (StringUtils.isEmpty(code)) {
            return ResponseBean.error("CODE 不能为空");
        }

        String openid;
        String sessionKey;
        if (code.equals("123")) {
            openid = "123";
            sessionKey = "123";

        } else {
            WxMaJscode2SessionResult sessionInfo = wxMaUitl.getOpenid(code);
            openid = sessionInfo.getOpenid();
            sessionKey = sessionInfo.getSessionKey();
        }


        TaPerson taPerson = taPersonService.getByOpenId(openid);
        if (null == taPerson) {
            taPerson = new TaPerson();
            taPerson.setOpenid(openid);
            taPerson.setStatus(Constants.STATUS_NORMAL);
            taPerson.setCreateDate(LocalDateTime.now());
            taPersonService.save(taPerson);
        }

        Map<String, Object> res = new HashMap<>();
        res.put("person", taPerson);
        res.put("sessionKey", sessionKey);

        return ResponseBean.success(res);
    }



    /**
     * 小程序
     *
     * @param taPerson
     * @return
     * @throws Exception
     */
    @PostMapping("/signup")
    @ApiOperation(value = "用户注册", notes = "用户注册", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean signup(@ApiParam("登录参数") @RequestBody TaPerson taPerson) throws Exception {
        if (StringUtils.isEmpty(taPerson.getOpenid())) {
            return ResponseBean.error("注册用户失败, 请退出重试");
        }

        TaPerson origin = taPersonService.getByOpenId(taPerson.getOpenid());
        if (null == origin) {
            taPerson.setStatus(Constants.STATUS_NORMAL);
            taPerson.setCreateDate(LocalDateTime.now());
            taPersonService.save(taPerson);

            return ResponseBean.success(taPerson);
        } else {
            taPerson.setPersonId(origin.getPersonId());
            taPersonService.updateById(taPerson);
        }

        // 获取最新的数据
        taPerson = taPersonService.getById(taPerson.getPersonId());

        Map<String, Object> res = sysLoginService.loginByPerson(taPerson);
        return ResponseBean.success(res);
    }

    /**
     * 系统人员登录
     *
     * @param param
     * @return
     * @throws Exception
     */
    @PostMapping("/signin")
    @ApiOperation(value = "用户登录", notes = "用户登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean signin(@ApiParam("登录参数") @RequestBody LoginParam param) throws Exception {

        if (StringUtils.isEmpty(param.getAccount()) || StringUtils.isEmpty(param.getPassword())) {
            return ResponseBean.error("登录用户或者密码不能为空");
        }

        SysUser user = sysUserService.login(param);
        List<String> dutyList = sysUserDutyService.getListByUser(user.getUserId());
        user.setDutyList(dutyList);

        TaPerson taPerson = null;
        if (StringUtils.isNotEmpty(user.getPhone())) {
            taPerson = taPersonService.getExistBy("phone", user.getPhone(), false, true);
        }

        // 登录新用户
        StpUtil.login(user.getUserId(), "miniapp");

        Map<String, Object> res = new HashMap<>();
        res.put("user", user);
        res.put("person", taPerson);
        res.put("token", StpUtil.getTokenValue());

        return ResponseBean.success(res);
    }

    /**
     * 获取当前用户
     *
     * @return
     * @throws Exception
     */
    @GetMapping("/current")
    @ApiOperation(value = "获取当前用户", notes = "获取当前用户", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean current() throws Exception {
        SysUser user = currentUser();
        List<String> dutyList = sysUserDutyService.getListByUser(user.getUserId());
        user.setDutyList(dutyList);

        TaPerson taPerson = null;
        if (StringUtils.isNotEmpty(user.getPhone())) {
            taPerson = taPersonService.getExistBy("phone", user.getPhone(), false, true);
        }

        // 登录新用户
        StpUtil.login(user.getUserId(), "miniapp");

        Map<String, Object> res = new HashMap<>();
        res.put("user", user);
        res.put("person", taPerson);
        res.put("token", StpUtil.getTokenValue());

        return ResponseBean.success(res);
    }

    @PutMapping("/auth-user")
    @ApiOperation(value = "授权头像", notes = "授权头像", httpMethod = "PUT", response = TaPerson.class)
    public ResponseBean updateUserInfo(@ApiParam("授权头像的参数") @RequestBody WxMaUitl.AuthParam param) throws Exception {
        // 解密用户信息
        WxMaUserInfo userInfo = wxMaUitl.getUserInfo(param);

        SysUser sysUser = currentUser();
        if (null == sysUser) {
            throw new Exception("校验当前人员失败, 请重试");
        }

        sysUser.setAvatar(userInfo.getAvatarUrl());
        sysUser.setName(userInfo.getNickName());
//        taPerson.setSex(Integer.parseInt(userInfo.getGender()));

//        taPersonService.updateById(sysUser);
         sysUserService.updateById(sysUser);

        return ResponseBean.success(sysUser);
    }

    @ApiOperation(value = "授权手机", notes = "授权手机", httpMethod = "PUT", response = TaPerson.class)
    @PutMapping("/auth-phone")
    public ResponseBean updateUserPhone(@ApiParam("授权手机的Code") @RequestParam String code) throws Exception {

        // 解密
        WxMaPhoneNumberInfo phoneNoInfo = wxMaUitl.getPhoneNoInfo(code);

        TaPerson taPerson = currentPerson();
        if (null == taPerson) {
            throw new Exception("校验当前人员失败, 请重试");
        }

        taPerson.setPhone(phoneNoInfo.getPhoneNumber());
        taPersonService.updateById(taPerson);

        Map<String, Object> res = sysLoginService.loginByPerson(taPerson);

        return ResponseBean.success(res);
    }

    @PostMapping("/change-password")
    @ApiOperation(value = "修改密码", notes = "修改密码", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean changePassword(@ApiParam("修改密码") @RequestBody WxChangePwd wxChangePwd) throws Exception {

        if (StringUtils.isEmpty(wxChangePwd.getPhone())
                || StringUtils.isEmpty(wxChangePwd.getCaptcha())
                || StringUtils.isEmpty(wxChangePwd.getPassword())
        ) {
            return ResponseBean.error("修改密码参数部分为空");
        }

        // TODO 校验验证码

        SysUser user = sysUserService.getExistBy("phone", wxChangePwd.getPhone(), false, true);
        if (null == user) {
            return ResponseBean.error("当前手机号不存在");
        }

        sysLoginService.changePassword(user, wxChangePwd);

        return ResponseBean.success("success");
    }

}
