package com.example.civilizedcity.message.impl;

import com.example.civilizedcity.message.IMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

@Slf4j
@Component
public class SMS implements IMessage {

    @Value("${message.sms.enable}")
    boolean enable;

    @Value("${message.sms.httpUrl}")
    String httpUrl;

    @Value("${message.sms.userName}")
    String userName;

    @Value("${message.sms.password}")
    String password;

    @Value("${message.sms.template}")
    String template;

    @Override
    public void send(String toUser, String code, String message) throws Exception {
        if (!enable) return;

        String content = String.format(template, message);

        String searchStr = "u=" + userName;
        searchStr += "&p=" + md5(password);
        searchStr += "&m=" + toUser;
        searchStr += "&c=" + encodeUrlString(content, "UTF-8");

        String resp = request(httpUrl, searchStr);
        log.debug(String.format("Send sms to %s , that result is : [ %s ]", toUser, resp));

        if (!"0".equals(resp)) {
            SMSError error = SMSError.getEnum(resp);
            String errMsg = null == error ? "未知错误" : error.getMessage();
            String errorMessage = String.format("Send sms to %s , that result is : [ %s ]: %s", toUser, resp, errMsg);
            throw new Exception(errorMessage);
        }
    }

    private String request(String httpUrl, String httpArg) {
        BufferedReader reader = null;
        String result = null;
        StringBuffer sbf = new StringBuffer();
        httpUrl = httpUrl + "?" + httpArg;

        try {
            URL url = new URL(httpUrl);
            HttpURLConnection connection = (HttpURLConnection) url.openConnection();
            connection.setRequestMethod("GET");
            connection.connect();
            InputStream is = connection.getInputStream();
            reader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
            String strRead = reader.readLine();
            if (strRead != null) {
                sbf.append(strRead);
                while ((strRead = reader.readLine()) != null) {
                    sbf.append("\n");
                    sbf.append(strRead);
                }
            }
            reader.close();
            result = sbf.toString();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    private String md5(String plainText) {
        StringBuffer buf = null;
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            md.update(plainText.getBytes());
            byte b[] = md.digest();
            int i;
            buf = new StringBuffer("");
            for (int offset = 0; offset < b.length; offset++) {
                i = b[offset];
                if (i < 0)
                    i += 256;
                if (i < 16)
                    buf.append("0");
                buf.append(Integer.toHexString(i));
            }
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        return buf.toString();
    }

    private String encodeUrlString(String str, String charset) {
        String strret = null;
        if (str == null)
            return str;
        try {
            strret = java.net.URLEncoder.encode(str, charset);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
        return strret;
    }
}
